@@ -580,6 +580,9 @@ Changes to files matching one of the regexps in this list are not listed.")
     "obsolete/options.el"
     "obsolete/old-whitespace.el"
     "obsolete/lucid.el"
+    "lisp/obsolete/fast-lock.el"
+    "lisp/obsolete/lazy-lock.el"
+    "lisp/obsolete/pc-mode.el"
     ;; ada-mode has been deleted, now in GNU ELPA
     "ada-mode.texi"
     "doc/misc/ada-mode.texi"
@@ -1595,6 +1598,22 @@ in the repository.")
     ("url-dired.el" . "url-dired.el")
     ("lisp/text-modes/tex-mode.el" . "tex-mode.el")
     ("editfns.c" . "editfns.c")
+    ("lisp/thumbs.el" . "thumbs.el")
+    ("lisp/linum.el" . "linum.el")
+    ("lisp/image-dired.el" . "image-dired.el")
+    ("lisp/url/url-about.el" . "url-about.el")
+    ("lisp/url/url-dired.el" . "url-dired.el")
+    ("lisp/ps-def.el" . "ps-def.el")
+    ("lisp/net/quickurl.el" . "quickurl.el")
+    ("lisp/vc/vc-mtn.el" . "vc-mtn.el")
+    ("lisp/mail/uce.el" . "uce.el")
+    ("test/lisp/progmodes/csharp-ts-mode-tests.el" . "csharp-mode.el")
+    ("lisp/makesum.el" . "makesum.el")
+    ("lisp/mh-e/mh-compat.el" . "mh-compat.el")
+    ("lisp/net/rlogin.el" . "rlogin.el")
+    ("lisp/emacs-lisp/autoload.el" . "autoload.el")
+    ("lisp/emacs-lisp/eieio-compat.el" . "eieio-compat.el")
+    ("autoarg.el" . "autoarg.el")
     )
   "Alist of files which have been renamed during their lifetime.
 Elements are (OLDNAME . NEWNAME).")
