@@ -1,10 +1,10 @@
 /* acl.c - access control lists
 
-   Copyright (C) 2002, 2008-2015 Free Software Foundation, Inc.
+   Copyright (C) 2002, 2008-2024 Free Software Foundation, Inc.
 
    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 3 of the License, or
+   the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
@@ -13,11 +13,13 @@
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
-   along with this program.  If not, see <http://www.gnu.org/licenses/>.
+   along with this program.  If not, see <https://www.gnu.org/licenses/>.
 
    Written by Paul Eggert.  */
 
-#include <stdbool.h>
+#ifndef _GL_ACL_H
+#define _GL_ACL_H 1
+
 #include <sys/types.h>
 #include <sys/stat.h>
 
@@ -28,3 +30,5 @@ int set_acl (char const *, int, mode_t);
 int qcopy_acl (char const *, int, char const *, int, mode_t);
 int copy_acl (char const *, int, char const *, int, mode_t);
 int chmod_or_fchmod (char const *, int, mode_t);
+
+#endif
