@@ -1,17 +1,15 @@
-;;; calc-embed.el --- embed Calc in a buffer
+;;; calc-embed.el --- embed Calc in a buffer  -*- lexical-binding:t -*-
 
-;; Copyright (C) 1990, 1991, 1992, 1993, 2001, 2002, 2003, 2004,
-;;   2005, 2006, 2007, 2008 Free Software Foundation, Inc.
+;; Copyright (C) 1990-1993, 2001-2024 Free Software Foundation, Inc.
 
 ;; Author: David Gillespie <daveg@synaptics.com>
-;; Maintainer: Jay Belanger <jay.p.belanger@gmail.com>
 
 ;; This file is part of GNU Emacs.
 
-;; GNU Emacs is free software; you can redistribute it and/or modify
+;; GNU Emacs is free software: you can redistribute it and/or modify
 ;; it under the terms of the GNU General Public License as published by
-;; the Free Software Foundation; either version 3, or (at your option)
-;; any later version.
+;; the Free Software Foundation, either version 3 of the License, or
+;; (at your option) any later version.
 
 ;; GNU Emacs is distributed in the hope that it will be useful,
 ;; but WITHOUT ANY WARRANTY; without even the implied warranty of
@@ -19,9 +17,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs; see the file COPYING.  If not, write to the
-;; Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
-;; Boston, MA 02110-1301, USA.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -31,6 +27,12 @@
 
 (require 'calc-ext)
 (require 'calc-macs)
+(require 'cl-lib)
+
+;; Declare functions which are defined elsewhere.
+(declare-function thing-at-point-looking-at "thingatpt"
+		  (regexp &optional distance))
+
 
 (defun calc-show-plain (n)
   (interactive "P")
@@ -44,25 +46,23 @@
 (defvar calc-embedded-modes nil)
 (defvar calc-embedded-globals nil)
 (defvar calc-embedded-active nil)
-(defvar calc-embedded-all-active nil)
-(make-variable-buffer-local 'calc-embedded-all-active)
-(defvar calc-embedded-some-active nil)
-(make-variable-buffer-local 'calc-embedded-some-active)
+(defvar-local calc-embedded-all-active nil)
+(defvar-local calc-embedded-some-active nil)
 
 ;; The following variables are customizable and defined in calc.el.
 (defvar calc-embedded-announce-formula)
 (defvar calc-embedded-open-formula)
 (defvar calc-embedded-close-formula)
-(defvar calc-embedded-open-word)
-(defvar calc-embedded-close-word)
 (defvar calc-embedded-open-plain)
 (defvar calc-embedded-close-plain)
 (defvar calc-embedded-open-new-formula)
 (defvar calc-embedded-close-new-formula)
 (defvar calc-embedded-open-mode)
 (defvar calc-embedded-close-mode)
+(defvar calc-embedded-word-regexp)
 
-(defconst calc-embedded-mode-vars '(("precision" . calc-internal-prec)
+(defconst calc-embedded-mode-vars '(("twos-complement" . calc-twos-complement-mode)
+                                    ("precision" . calc-internal-prec)
 				    ("word-size" . calc-word-size)
 				    ("angles" . calc-angle-mode)
 				    ("symbolic" . calc-symbolic-mode)
@@ -96,35 +96,35 @@
 ))
 
 
-;;; Format of calc-embedded-info vector:
-;;;    0   Editing buffer.
-;;;    1   Calculator buffer.
-;;;    2   Top of current formula (marker).
-;;;    3   Bottom of current formula (marker).
-;;;    4   Top of current formula's delimiters (marker).
-;;;    5   Bottom of current formula's delimiters (marker).
-;;;    6   String representation of current formula.
-;;;    7   Non-nil if formula is embedded within a single line.
-;;;    8   Internal representation of current formula.
-;;;    9   Variable assigned by this formula, or nil.
-;;;   10   List of variables upon which this formula depends.
-;;;   11   Evaluated value of the formula, or nil.
-;;;   12   Mode settings for current formula.
-;;;   13   Local mode settings for current formula.
-;;;   14   Permanent mode settings for current formula.
-;;;   15   Global mode settings for editing buffer.
+;; Format of calc-embedded-info vector:
+;;    0   Editing buffer.
+;;    1   Calculator buffer.
+;;    2   Top of current formula (marker).
+;;    3   Bottom of current formula (marker).
+;;    4   Top of current formula's delimiters (marker).
+;;    5   Bottom of current formula's delimiters (marker).
+;;    6   String representation of current formula.
+;;    7   Non-nil if formula is embedded within a single line.
+;;    8   Internal representation of current formula.
+;;    9   Variable assigned by this formula, or nil.
+;;   10   List of variables upon which this formula depends.
+;;   11   Evaluated value of the formula, or nil.
+;;   12   Mode settings for current formula.
+;;   13   Local mode settings for current formula.
+;;   14   Permanent mode settings for current formula.
+;;   15   Global mode settings for editing buffer.
 
 
-;;; calc-embedded-active is an a-list keyed on buffers; each cdr is a
-;;; sorted list of calc-embedded-infos in that buffer.  We do this
-;;; rather than using buffer-local variables because the latter are
-;;; thrown away when a buffer changes major modes.
+;; calc-embedded-active is an a-list keyed on buffers; each cdr is a
+;; sorted list of calc-embedded-infos in that buffer.  We do this
+;; rather than using buffer-local variables because the latter are
+;; thrown away when a buffer changes major modes.
 
 (defvar calc-embedded-original-modes nil
   "The mode settings for Calc buffer when put in embedded mode.")
 
 (defun calc-embedded-save-original-modes ()
-  "Save the current Calc modes when entereding embedded mode."
+  "Save the current Calc modes when entering embedded mode."
   (let ((calcbuf (save-excursion
                    (calc-create-buffer)
                    (current-buffer)))
@@ -143,7 +143,7 @@
               (let ((var (cdr (car v))))
                 (unless (memq var '(the-language the-display-just))
                   (setq modes
-                        (cons (cons var (symbol-value var)) 
+                        (cons (cons var (symbol-value var))
                               modes))))
               (setq v (cdr v))))
           (setq calc-embedded-original-modes (cons lang modes)))
@@ -158,10 +158,9 @@
         (message "Current modes will be preserved when leaving embedded mode."))
     (message "Not in embedded mode.")))
 
-(defun calc-embedded-restore-original-modes ()
+(defun calc-embedded-restore-original-modes (calcbuf)
   "Restore the original Calc modes when leaving embedded mode."
-  (let ((calcbuf (get-buffer "*Calculator*"))
-        (changed nil)
+  (let ((changed nil)
         (lang (car calc-embedded-original-modes))
         (modes (cdr calc-embedded-original-modes)))
     (if (and calcbuf calc-embedded-original-modes)
@@ -182,7 +181,7 @@
             (calc-set-mode-line))))
     (setq calc-embedded-original-modes nil)))
 
-;; The variables calc-embed-outer-top, calc-embed-outer-bot, 
+;; The variables calc-embed-outer-top, calc-embed-outer-bot,
 ;; calc-embed-top and calc-embed-bot are
 ;; local to calc-do-embedded, calc-embedded-mark-formula,
 ;; calc-embedded-duplicate, calc-embedded-new-formula and
@@ -195,7 +194,7 @@
 
 ;; The variable calc-embed-arg is local to calc-do-embedded,
 ;; calc-embedded-update-formula, calc-embedded-edit and
-;; calc-do-embedded-activate, but is used by 
+;; calc-do-embedded-activate, but is used by
 ;; calc-embedded-make-info, which is called by the above
 ;; functions.
 (defvar calc-embed-arg)
@@ -208,32 +207,35 @@
 
 ;; The following is to take care of any minor modes which override
 ;; a Calc command.
-(defvar calc-override-minor-modes-map 
-  (make-sparse-keymap)
-  "A list of keybindings that might be overwritten by minor modes.")
+(defvar-keymap calc-override-minor-modes-map
+  :doc "A list of keybindings that might be overwritten by minor modes.")
 
 ;; Add any keys that might be overwritten here.
 (define-key calc-override-minor-modes-map "`" 'calc-edit)
 
-(defvar calc-override-minor-modes 
+(defvar calc-override-minor-modes
   (cons t calc-override-minor-modes-map))
 
-(defun calc-do-embedded (calc-embed-arg end obeg oend)
+(defvar calc-embedded-no-reselect nil)
+
+(defun calc-do-embedded (embed-arg end obeg oend)
+  (let ((calc-embed-arg embed-arg))
   (if calc-embedded-info
 
       ;; Turn embedded mode off or switch to a new buffer.
       (cond ((eq (current-buffer) (aref calc-embedded-info 1))
 	     (let ((calcbuf (current-buffer))
-		   (buf (aref calc-embedded-info 0)))
+		   ;; (buf (aref calc-embedded-info 0))
+		   )
 	       (calc-embedded-original-buffer t)
 	       (calc-embedded nil)
 	       (switch-to-buffer calcbuf)))
 
 	    ((eq (current-buffer) (aref calc-embedded-info 0))
 	     (let* ((info calc-embedded-info)
-		    (mode calc-embedded-modes))
-	       (save-excursion
-		 (set-buffer (aref info 1))
+		    (mode calc-embedded-modes)
+                    (calcbuf (aref calc-embedded-info 1)))
+	       (with-current-buffer (aref info 1)
 		 (if (and (> (calc-stack-size) 0)
 			  (equal (calc-top 1 'full) (aref info 8)))
 		     (let ((calc-no-refresh-evaltos t))
@@ -252,14 +254,13 @@
                (setq minor-mode-overriding-map-alist
                      (remq calc-override-minor-modes minor-mode-overriding-map-alist))
 	       (set-buffer-modified-p (buffer-modified-p))
-               (calc-embedded-restore-original-modes)
+               (calc-embedded-restore-original-modes calcbuf)
 	       (or calc-embedded-quiet
-		   (message "Back to %s mode" mode-name))))
+		   (message "Back to %s mode" (format-mode-line mode-name)))))
 
 	    (t
 	     (if (buffer-name (aref calc-embedded-info 0))
-		 (save-excursion
-		   (set-buffer (aref calc-embedded-info 0))
+		 (with-current-buffer (aref calc-embedded-info 0)
 		   (or (y-or-n-p (format "Cancel Calc Embedded mode in buffer %s? "
 					 (buffer-name)))
 		       (keyboard-quit))
@@ -280,7 +281,7 @@
       (calc-embedded-save-original-modes)
       (or calc-embedded-globals
 	  (calc-find-globals))
-      (setq info 
+      (setq info
             (calc-embedded-make-info (point) nil t calc-embed-arg end obeg oend))
       (if (eq (car-safe (aref info 8)) 'error)
 	  (progn
@@ -291,7 +292,7 @@
 	    (calc-embedded-info info)
 	    (calc-embedded-no-reselect t))
 	(calc-wrapper
-	 (let* ((okay nil)
+	 (let* (;; (okay nil)
 		(calc-no-refresh-evaltos t))
 	   (if (aref info 8)
                (progn
@@ -330,11 +331,14 @@
       (if calc-embedded-firsttime-formula
           (run-hooks 'calc-embedded-new-formula-hook))
       (or (eq calc-embedded-quiet t)
-	  (message "Embedded Calc mode enabled; %s to return to normal"
-		   (if calc-embedded-quiet
-		       "Type `C-x * x'"
-		     "Give this command again")))))
-  (scroll-down 0))    ; fix a bug which occurs when truncate-lines is changed.
+	  (message (concat
+                    "Embedded Calc mode enabled; "
+                    (if calc-embedded-quiet
+                        (substitute-command-keys
+                         "Type \\`C-x * x'")
+                      "Give this command again")
+                    " to return to normal")))))
+  (scroll-down 0)))    ; fix a bug which occurs when truncate-lines is changed.
 
 
 (defun calc-embedded-select (arg)
@@ -351,9 +355,10 @@
        (calc-select-part 2)))
 
 
-(defun calc-embedded-update-formula (calc-embed-arg)
+(defun calc-embedded-update-formula (embed-arg)
   (interactive "P")
-  (if calc-embed-arg
+  (let ((calc-embed-arg embed-arg))
+  (if embed-arg
       (let ((entry (assq (current-buffer) calc-embedded-active)))
 	(while (setq entry (cdr entry))
 	  (and (eq (car-safe (aref (car entry) 8)) 'calcFunc-evalto)
@@ -374,12 +379,13 @@
 	    (progn
 	      (save-excursion
 		(calc-embedded-update info 14 'eval t))
-	      (goto-char (+ (aref info 4) pt))))))))
+	      (goto-char (+ (aref info 4) pt)))))))))
 
 
-(defun calc-embedded-edit (calc-embed-arg)
+(defun calc-embedded-edit (embed-arg)
   (interactive "P")
-  (let ((info (calc-embedded-make-info (point) nil t calc-embed-arg))
+  (let ((calc-embed-arg embed-arg))
+  (let ((info (calc-embedded-make-info (point) nil t embed-arg))
 	str)
     (if (eq (car-safe (aref info 8)) 'error)
 	(progn
@@ -388,22 +394,20 @@
     (calc-wrapper
      (setq str (math-showing-full-precision
 		(math-format-nice-expr (aref info 8) (frame-width))))
-     (calc-edit-mode (list 'calc-embedded-finish-edit info))
+     (calc--edit-mode (lambda () (calc-embedded-finish-edit info)))
      (insert str "\n")))
-  (calc-show-edit-buffer))
+  (calc-show-edit-buffer)))
 
 (defvar calc-original-buffer)
 (defvar calc-edit-top)
 (defun calc-embedded-finish-edit (info)
   (let ((buf (current-buffer))
 	(str (buffer-substring calc-edit-top (point-max)))
-	(start (point))
-	pos)
+	(start (point))) ;; pos
     (switch-to-buffer calc-original-buffer)
-    (let ((val (save-excursion
-		 (set-buffer (aref info 1))
+    (let ((val (with-current-buffer (aref info 1)
 		 (let ((calc-language nil)
-		       (math-expr-opers math-standard-opers))
+		       (math-expr-opers (math-standard-ops)))
 		   (math-read-expr str)))))
       (if (eq (car-safe val) 'error)
 	  (progn
@@ -414,7 +418,9 @@
       (aset info 8 val)
       (calc-embedded-update info 14 t t))))
 
-(defun calc-do-embedded-activate (calc-embed-arg cbuf)
+;;;###autoload
+(defun calc-do-embedded-activate (embed-arg cbuf)
+  (let ((calc-embed-arg embed-arg))
   (calc-plain-buffer-only)
   (if calc-embed-arg
       (calc-embedded-forget))
@@ -441,7 +447,7 @@
 	  (or (eq (car-safe (aref info 8)) 'error)
 	      (goto-char (aref info 5))))))
     (message "Activating %s for Calc Embedded mode...done" (buffer-name)))
-  (calc-embedded-active-state t))
+  (calc-embedded-active-state t)))
 
 (defun calc-plain-buffer-only ()
   (if (memq major-mode '(calc-mode calc-trail-mode calc-edit-mode))
@@ -565,7 +571,7 @@ The command \\[yank] can retrieve it from there."
     (goto-char calc-embed-outer-bot)
     (insert "\n")
     (setq new-top (point))
-    (insert-buffer-substring (current-buffer) 
+    (insert-buffer-substring (current-buffer)
                              calc-embed-outer-top calc-embed-outer-bot)
     (goto-char (+ new-top (- calc-embed-top calc-embed-outer-top)))
     (let ((calc-embedded-quiet (if already t 'x)))
@@ -643,6 +649,8 @@ The command \\[yank] can retrieve it from there."
 (defvar calc-embed-prev-modes)
 
 (defun calc-embedded-set-modes (gmodes modes local-modes &optional temp)
+  (defvar the-language)
+  (defvar the-display-just)
   (let ((the-language (calc-embedded-language))
 	(the-display-just (calc-embedded-justify))
 	(v gmodes)
@@ -660,7 +668,7 @@ The command \\[yank] can retrieve it from there."
 	    (equal (symbol-value (car (car v))) value))
 	  (progn
 	    (setq changed t)
-	    (if temp (setq calc-embed-prev-modes 
+	    (if temp (setq calc-embed-prev-modes
                            (cons (cons (car (car v))
                                        (symbol-value (car (car v))))
                                  calc-embed-prev-modes)))
@@ -733,13 +741,13 @@ The command \\[yank] can retrieve it from there."
 
 (defun calc-find-globals ()
   (interactive)
-  (and (eq major-mode 'calc-mode)
+  (and (derived-mode-p 'calc-mode)
        (error "This command should be used in a normal editing buffer"))
   (make-local-variable 'calc-embedded-globals)
   (let ((case-fold-search nil)
 	(modes nil)
 	(save-pt (point))
-	found value)
+	found) ;; value
     (goto-char (point-min))
     (while (re-search-forward "\\[calc-global-mode: *\\([-a-z]+\\): *\\(\"\\([^\"\n\\]\\|\\\\.\\)*\"\\|[- ()a-zA-Z0-9]+\\)\\]" nil t)
       (and (setq found (assoc (buffer-substring (match-beginning 1)
@@ -762,7 +770,7 @@ The command \\[yank] can retrieve it from there."
 	(modes nil)
 	(emodes nil)
 	(pmodes nil)
-	found value)
+	found) ;; value
     (while (and no-defaults (search-backward "[calc-" nil t))
       (forward-char 6)
       (or (and (looking-at "mode: *\\([-a-z]+\\): *\\(\"\\([^\"\n\\]\\|\\\\.\\)*\"\\|[- ()a-zA-Z0-9]+\\)]")
@@ -803,23 +811,27 @@ The command \\[yank] can retrieve it from there."
       (backward-char 6))
     (goto-char save-pt)
     (unless (assq 'the-language modes)
-      (let ((lang (assoc major-mode calc-language-alist)))
+      (let ((lang (cl-assoc-if #'derived-mode-p calc-language-alist)))
         (if lang
             (setq modes (cons (cons 'the-language (cdr lang))
                               modes)))))
     (list modes emodes pmodes)))
 
 ;; The variable calc-embed-vars-used is local to calc-embedded-make-info,
-;; calc-embedded-evaluate-expr and calc-embedded-update, but is 
+;; calc-embedded-evaluate-expr and calc-embedded-update, but is
 ;; used by calc-embedded-find-vars, which is called by the above functions.
 (defvar calc-embed-vars-used)
 
 (defun calc-embedded-make-info (point cbuf fresh &optional
-				      calc-embed-top calc-embed-bot 
-                                      calc-embed-outer-top calc-embed-outer-bot)
-  (let* ((bufentry (assq (current-buffer) calc-embedded-active))
+				      embed-top embed-bot
+                                      embed-outer-top embed-outer-bot)
+  (let* ((calc-embed-top embed-top)
+	 (calc-embed-bot embed-bot)
+	 (calc-embed-outer-top embed-outer-top)
+	 (calc-embed-outer-bot embed-outer-bot)
+	 (bufentry (assq (current-buffer) calc-embedded-active))
 	 (found bufentry)
-	 (force (and fresh calc-embed-top))
+	 (force (and fresh calc-embed-top (null (equal calc-embed-top '(t)))))
 	 (fixed calc-embed-top)
 	 (new-info nil)
 	 info str)
@@ -828,41 +840,35 @@ The command \\[yank] can retrieve it from there."
          (setq found (list (current-buffer))
                calc-embedded-active (cons found calc-embedded-active)
                calc-embedded-firsttime-buf t)
-         (let ((newann (assoc major-mode calc-embedded-announce-formula-alist))
-               (newform (assoc major-mode calc-embedded-open-close-formula-alist))
-               (newword (assoc major-mode calc-embedded-open-close-word-alist))
-               (newplain (assoc major-mode calc-embedded-open-close-plain-alist))
-               (newnewform 
-                (assoc major-mode calc-embedded-open-close-new-formula-alist))
-               (newmode (assoc major-mode calc-embedded-open-close-mode-alist)))
+         (let ((newann (cl-assoc-if #'derived-mode-p
+                                    calc-embedded-announce-formula-alist))
+               (newform (cl-assoc-if #'derived-mode-p
+                                     calc-embedded-open-close-formula-alist))
+               (newword (cl-assoc-if #'derived-mode-p
+                                     calc-embedded-word-regexp-alist))
+               (newplain (cl-assoc-if #'derived-mode-p
+                                      calc-embedded-open-close-plain-alist))
+               (newnewform
+                (cl-assoc-if #'derived-mode-p
+                             calc-embedded-open-close-new-formula-alist))
+               (newmode (cl-assoc-if #'derived-mode-p
+                                     calc-embedded-open-close-mode-alist)))
            (when newann
-             (make-local-variable 'calc-embedded-announce-formula)
-             (setq calc-embedded-announce-formula (cdr newann)))
+             (setq-local calc-embedded-announce-formula (cdr newann)))
            (when newform
-             (make-local-variable 'calc-embedded-open-formula)
-             (make-local-variable 'calc-embedded-close-formula)
-             (setq calc-embedded-open-formula (nth 0 (cdr newform)))
-             (setq calc-embedded-close-formula (nth 1 (cdr newform))))
+             (setq-local calc-embedded-open-formula (nth 0 (cdr newform)))
+             (setq-local calc-embedded-close-formula (nth 1 (cdr newform))))
            (when newword
-             (make-local-variable 'calc-embedded-open-word)
-             (make-local-variable 'calc-embedded-close-word)
-             (setq calc-embedded-open-word (nth 0 (cdr newword)))
-             (setq calc-embedded-close-word (nth 1 (cdr newword))))
+             (setq-local calc-embedded-word-regexp (nth 1 newword)))
            (when newplain
-             (make-local-variable 'calc-embedded-open-plain)
-             (make-local-variable 'calc-embedded-close-plain)
-             (setq calc-embedded-open-plain (nth 0 (cdr newplain)))
-             (setq calc-embedded-close-plain (nth 1 (cdr newplain))))
+             (setq-local calc-embedded-open-plain (nth 0 (cdr newplain)))
+             (setq-local calc-embedded-close-plain (nth 1 (cdr newplain))))
            (when newnewform
-             (make-local-variable 'calc-embedded-open-new-formula)
-             (make-local-variable 'calc-embedded-close-new-formula)
-             (setq calc-embedded-open-new-formula (nth 0 (cdr newnewform)))
-             (setq calc-embedded-close-new-formula (nth 1 (cdr newnewform))))
+             (setq-local calc-embedded-open-new-formula (nth 0 (cdr newnewform)))
+             (setq-local calc-embedded-close-new-formula (nth 1 (cdr newnewform))))
            (when newmode
-             (make-local-variable 'calc-embedded-open-mode)
-             (make-local-variable 'calc-embedded-close-mode)
-             (setq calc-embedded-open-mode (nth 0 (cdr newmode)))
-             (setq calc-embedded-close-mode (nth 1 (cdr newmode)))))))
+             (setq-local calc-embedded-open-mode (nth 0 (cdr newmode)))
+             (setq-local calc-embedded-close-mode (nth 1 (cdr newmode)))))))
     (while (and (cdr found)
 		(> point (aref (car (cdr found)) 3)))
       (setq found (cdr found)))
@@ -877,9 +883,9 @@ The command \\[yank] can retrieve it from there."
       (aset info 1 (or cbuf (save-excursion
 			      (calc-create-buffer)
 			      (current-buffer)))))
-    (if (and 
+    (if (and
          (or (integerp calc-embed-top) (equal calc-embed-top '(4)))
-         (not calc-embed-bot))  
+         (not calc-embed-bot))
                                         ; started with a user-supplied argument
 	(progn
           (if (equal calc-embed-top '(4))
@@ -903,9 +909,18 @@ The command \\[yank] can retrieve it from there."
 	  (setq calc-embed-top (aref info 2)
 		fixed calc-embed-top)
 	(if (consp calc-embed-top)
-	    (let ((calc-embedded-open-formula calc-embedded-open-word)
-		  (calc-embedded-close-formula calc-embedded-close-word))
-	      (calc-embedded-find-bounds 'plain))
+            (progn
+              (require 'thingatpt)
+              (if (thing-at-point-looking-at calc-embedded-word-regexp)
+                  (progn
+                    (setq calc-embed-top (copy-marker (match-beginning 0)))
+                    (setq calc-embed-bot (copy-marker (match-end 0)))
+                    (setq calc-embed-outer-top calc-embed-top)
+                    (setq calc-embed-outer-bot calc-embed-bot))
+                (setq calc-embed-top (point-marker))
+                (setq calc-embed-bot (point-marker))
+                (setq calc-embed-outer-top calc-embed-top)
+                (setq calc-embed-outer-bot calc-embed-bot)))
 	  (or calc-embed-top
 	      (calc-embedded-find-bounds 'plain)))
 	(aset info 2 (copy-marker (min calc-embed-top calc-embed-bot)))
@@ -937,8 +952,7 @@ The command \\[yank] can retrieve it from there."
 		 (pref-len (length open-plain))
 		 (calc-embed-vars-used nil)
 		 suff-pos val temp)
-	    (save-excursion
-	      (set-buffer (aref info 1))
+	    (with-current-buffer (aref info 1)
 	      (calc-embedded-set-modes (aref info 15)
 				       (aref info 12) (aref info 14))
 	      (if (and (> (length str) pref-len)
@@ -1152,7 +1166,7 @@ The command \\[yank] can retrieve it from there."
 	       (if need-display
 		   (calc-embedded-set-justify (cdr (car calc-embed-prev-modes)))))
 	      (t
-	       (set (car (car calc-embed-prev-modes)) 
+	       (set (car (car calc-embed-prev-modes))
                     (cdr (car calc-embed-prev-modes)))))
 	(setq calc-embed-prev-modes (cdr calc-embed-prev-modes))))))
 
@@ -1161,7 +1175,6 @@ The command \\[yank] can retrieve it from there."
 
 ;;; These are hooks called by the main part of Calc.
 
-(defvar calc-embedded-no-reselect nil)
 (defun calc-embedded-select-buffer ()
   (if (eq (current-buffer) (aref calc-embedded-info 0))
       (let ((info calc-embedded-info)
@@ -1195,8 +1208,7 @@ The command \\[yank] can retrieve it from there."
 (defun calc-embedded-finish-command ()
   (let ((buf (current-buffer))
 	horiz vert)
-    (save-excursion
-      (set-buffer (aref calc-embedded-info 1))
+    (with-current-buffer (aref calc-embedded-info 1)
       (if (> (calc-stack-size) 0)
 	  (let ((pt (point))
 		(col (current-column))
@@ -1224,11 +1236,10 @@ The command \\[yank] can retrieve it from there."
 
 (defun calc-embedded-stack-change ()
   (or calc-executing-macro
-      (save-excursion
-	(set-buffer (aref calc-embedded-info 1))
+      (with-current-buffer (aref calc-embedded-info 1)
 	(let* ((info calc-embedded-info)
 	       (extra-line (if (eq calc-language 'big) 1 0))
-	       (the-point (point))
+	       ;; (the-point (point))
 	       (empty (= (calc-stack-size) 0))
 	       (entry (if empty
 			  (list '(var empty var-empty) 1 nil)
@@ -1262,6 +1273,7 @@ The command \\[yank] can retrieve it from there."
       (set-buffer-modified-p (buffer-modified-p)))))
 
 (defun calc-embedded-modes-change (vars)
+  (defvar the-language) (defvar the-display-just)
   (if (eq (car vars) 'calc-language) (setq vars '(the-language)))
   (if (eq (car vars) 'calc-display-just) (setq vars '(the-display-just)))
   (while (and vars
@@ -1374,5 +1386,8 @@ The command \\[yank] can retrieve it from there."
 
 (provide 'calc-embed)
 
-;;; arch-tag: 1b8f311e-fba1-40d3-b8c3-1d6f68fd26fc
+;; Local variables:
+;; generated-autoload-file: "calc-loaddefs.el"
+;; End:
+
 ;;; calc-embed.el ends here
