@@ -1,10 +1,11 @@
-;;; rmailout.el --- "RMAIL" mail reader for Emacs: output message to a file
+;;; rmailout.el --- "RMAIL" mail reader for Emacs: output message to a file  -*- lexical-binding: t; -*-
 
-;; Copyright (C) 1985, 1987, 1993, 1994, 2001, 2002, 2003, 2004, 2005,
-;;   2006, 2007, 2008, 2009, 2010  Free Software Foundation, Inc.
+;; Copyright (C) 1985, 1987, 1993-1994, 2001-2024 Free Software
+;; Foundation, Inc.
 
-;; Maintainer: FSF
+;; Maintainer: emacs-devel@gnu.org
 ;; Keywords: mail
+;; Package: rmail
 
 ;; This file is part of GNU Emacs.
 
@@ -19,7 +20,7 @@
 ;; GNU General Public License for more details.
 
 ;; You should have received a copy of the GNU General Public License
-;; along with GNU Emacs.  If not, see <http://www.gnu.org/licenses/>.
+;; along with GNU Emacs.  If not, see <https://www.gnu.org/licenses/>.
 
 ;;; Commentary:
 
@@ -33,7 +34,6 @@
   :type 'boolean
   :group 'rmail-output)
 
-;; FIXME risky?
 (defcustom rmail-output-file-alist nil
   "Alist matching regexps to suggested output Rmail files.
 This is a list of elements of the form (REGEXP . NAME-EXP).
@@ -46,6 +46,7 @@ a file name as a string."
 			       (string :tag "File Name")
 			       sexp)))
   :group 'rmail-output)
+;; This is risky because NAME-EXP gets evalled.
 ;;;###autoload(put 'rmail-output-file-alist 'risky-local-variable t)
 
 (defcustom rmail-fields-not-to-output nil
@@ -55,37 +56,69 @@ The function `rmail-delete-unwanted-fields' uses this, ignoring case."
 		 regexp)
   :group 'rmail-output)
 
+(defcustom rmail-output-reset-deleted-flag nil
+  "Non-nil means reset the \"deleted\" flag when outputting a message to a file."
+  :type '(choice (const :tag "Output with the \"deleted\" flag reset" t)
+                 (const :tag "Output with the \"deleted\" flag intact" nil))
+  :version "27.1"
+  :group 'rmail-output)
+
 (defun rmail-output-read-file-name ()
   "Read the file name to use for `rmail-output'.
-Set `rmail-default-file' to this name as well as returning it."
-  (let ((default-file
-	  (let (answer tail)
-	    (setq tail rmail-output-file-alist)
-	    ;; Suggest a file based on a pattern match.
-	    (while (and tail (not answer))
-	      (save-excursion
-		(goto-char (point-min))
-		(if (re-search-forward (car (car tail)) nil t)
-		    (setq answer (eval (cdr (car tail)))))
-		(setq tail (cdr tail))))
+Set `rmail-default-file' to this name as well as returning it.
+This uses `rmail-output-file-alist'."
+  (let* ((default-file
+	   (or
+	    (when rmail-output-file-alist
+	      (or rmail-buffer (error "There is no Rmail buffer"))
+	      (save-current-buffer
+		(set-buffer rmail-buffer)
+		(let ((beg (rmail-msgbeg rmail-current-message))
+		      (end (rmail-msgend rmail-current-message)))
+		  (if (rmail-buffers-swapped-p) (set-buffer rmail-view-buffer))
+		  (save-excursion
+		    (save-restriction
+		      (widen)
+		      (narrow-to-region beg end)
+		      (let ((tail rmail-output-file-alist)
+			    answer) ;; err
+			;; Suggest a file based on a pattern match.
+			(while (and tail (not answer))
+			  (goto-char (point-min))
+			  (if (re-search-forward (caar tail) nil t)
+			      (setq answer
+				    (condition-case err
+					(eval (cdar tail) t)
+				      (error
+				       (display-warning
+					'rmail-output
+					(format-message "\
+Error evaluating `rmail-output-file-alist' element:
+regexp: %s
+action: %s
+error: %S\n"
+						(caar tail) (cdar tail) err)
+                                        :error)
+				       nil))))
+			  (setq tail (cdr tail)))
+			answer))))))
 	    ;; If no suggestion, use same file as last time.
-	    (or answer rmail-default-file))))
-    (let ((read-file
-	   (expand-file-name
-	    (read-file-name
-	     (concat "Output message to mail file (default "
-		     (file-name-nondirectory default-file)
-		     "): ")
-	     (file-name-directory default-file)
-	     (abbreviate-file-name default-file))
-	    (file-name-directory default-file))))
-      (setq rmail-default-file
-	    (if (file-directory-p read-file)
-		(expand-file-name (file-name-nondirectory default-file)
-				  read-file)
-	      (expand-file-name
-	       (or read-file (file-name-nondirectory default-file))
-	       (file-name-directory default-file)))))))
+	    rmail-default-file))
+	 (read-file
+	  (expand-file-name
+	   (read-file-name
+            (format-prompt "Output message to mail file"
+                           (file-name-nondirectory default-file))
+	    (file-name-directory default-file)
+	    (abbreviate-file-name default-file))
+	   (file-name-directory default-file))))
+    (setq rmail-default-file
+	  (if (file-directory-p read-file)
+	      (expand-file-name (file-name-nondirectory default-file)
+				read-file)
+	    (expand-file-name
+	     (or read-file (file-name-nondirectory default-file))
+	     (file-name-directory default-file))))))
 
 (defun rmail-delete-unwanted-fields (preserve)
   "Delete all headers matching `rmail-fields-not-to-output'.
@@ -163,7 +196,8 @@ display message number MSG."
 
 (defun rmail-convert-to-babyl-format ()
   "Convert the mbox message in the current buffer to Babyl format."
-  (let ((count 0) (start (point-min))
+  (let (;; (count 0)
+	(start (point-min))
 	(case-fold-search nil)
 	(buffer-undo-list t))
     (goto-char (point-min))
@@ -322,7 +356,8 @@ the text directly to FILE-NAME, and displays a \"Wrote file\" message
 unless NOMSG is a symbol (neither nil nor t).
 AS-SEEN is non-nil if we are copying the message \"as seen\"."
   (let ((case-fold-search t)
-	from date)
+        encrypted-file-name
+	) ;; from date
     (goto-char (point-min))
     ;; Preserve the Mail-From and MIME-Version fields
     ;; even if they have been pruned.
@@ -341,10 +376,45 @@ AS-SEEN is non-nil if we are copying the message \"as seen\"."
     (goto-char (point-min))
     (let ((buf (find-buffer-visiting file-name))
 	  (tembuf (current-buffer)))
+      (when (string-match "[.]gpg\\'" file-name)
+        (setq encrypted-file-name file-name
+              file-name (substring file-name 0 (match-beginning 0))))
       (if (null buf)
-	  (let ((coding-system-for-write 'raw-text-unix))
+	  (let ((coding-system-for-write 'raw-text-unix)
+                (coding-system-for-read 'raw-text-unix))
+            ;; If the specified file is encrypted, decrypt it.
+            (when encrypted-file-name
+              (with-temp-buffer
+                (insert-file-contents encrypted-file-name)
+                (write-region 1 (point-max) file-name nil 'nomsg)))
 	    ;; FIXME should ensure existing file ends with a blank line.
-	    (write-region (point-min) (point-max) file-name t nomsg))
+	    (write-region (point-min) (point-max) file-name t
+                          (if (or nomsg encrypted-file-name)
+                              'nomsg))
+            ;; If the specified file was encrypted, re-encrypt it.
+            (when encrypted-file-name
+              ;; Save the old encrypted file as a backup.
+              (rename-file encrypted-file-name
+                           (make-backup-file-name encrypted-file-name)
+                           t)
+              (if (= 0
+                     (call-process "gpg" nil nil
+                                   "--use-agent" "--batch" "--no-tty"
+                                   "--encrypt" "-r"
+                                   user-mail-address
+                                   file-name))
+                  ;; Delete the unencrypted file if encryption succeeded.
+                  (delete-file file-name)
+                ;; If encrypting failed, put back the original
+                ;; encrypted file and signal an error.
+                (rename-file (make-backup-file-name encrypted-file-name)
+                             encrypted-file-name
+                             t)
+                (error "Encryption failed; %s unchanged"
+                       encrypted-file-name))
+              (unless nomsg
+                (message "Added to %s" encrypted-file-name)))
+            )
 	(if (eq buf (current-buffer))
 	    (error "Can't output message to same file it's already in"))
 	;; File has been visited, in buffer BUF.
@@ -363,7 +433,7 @@ AS-SEEN is non-nil if we are copying the message \"as seen\"."
 
 (defun rmail-output-to-rmail-buffer (tembuf msg)
   "Copy message in TEMBUF into the current Rmail buffer.
-Do what is necessary to make Rmail know about the new message. then
+Do what is necessary to make Rmail know about the new message, then
 display message number MSG."
   (save-excursion
     (rmail-swap-buffers-maybe)
@@ -376,14 +446,15 @@ display message number MSG."
     (rmail-maybe-set-message-counters)
     ;; Insert the new message after the last old message.
     (widen)
-    ;; Make sure the last old message ends with a blank line.
-    (goto-char (point-max))
-    (rmail-ensure-blank-line)
-    ;; Insert the new message at the end.
-    (narrow-to-region (point-max) (point-max))
+    (unless (zerop (buffer-size))
+      ;; Make sure the last old message ends with a blank line.
+      (goto-char (point-max))
+      (rmail-ensure-blank-line)
+      ;; Insert the new message at the end.
+      (narrow-to-region (point-max) (point-max)))
     (insert-buffer-substring tembuf)
     (rmail-count-new-messages t)
-    ;; FIXME should re-use existing windows.
+    ;; FIXME should reuse existing windows.
     (if (rmail-summary-exists)
 	(rmail-select-summary (rmail-update-summary)))
     (rmail-show-message-1 msg)))
@@ -408,9 +479,15 @@ buffer, updates it accordingly.
 This command always outputs the complete message header, even if
 the header display is currently pruned.
 
+If `rmail-output-reset-deleted-flag' is non-nil, the message's
+deleted flag is reset in the message appended to the destination
+file.  Otherwise, the appended message will remain marked as
+deleted if it was deleted before invoking this command.
+
 Optional prefix argument COUNT (default 1) says to output that
 many consecutive messages, starting with the current one (ignoring
-deleted messages).  If `rmail-delete-after-output' is non-nil, deletes
+deleted messages, unless `rmail-output-reset-deleted-flag' is
+non-nil).  If `rmail-delete-after-output' is non-nil, deletes
 messages after output.
 
 The optional third argument NOATTRIBUTE, if non-nil, says not to
@@ -466,31 +543,50 @@ from a non-Rmail buffer.  In this case, COUNT is ignored."
       (if rmail-buffer
 	  (set-buffer rmail-buffer)
 	(error "There is no Rmail buffer"))
+      (if (zerop rmail-total-messages)
+	  (error "No messages to output"))
       (let ((orig-count count)
-	    beg end)
+	    beg end delete-attr-reset-p)
 	(while (> count 0)
-	  (setq beg (rmail-msgbeg rmail-current-message)
-		end (rmail-msgend rmail-current-message))
-	  ;; All access to the buffer's local variables is now finished...
-	  (save-excursion
-	    ;; ... so it is ok to go to a different buffer.
-	    (if (rmail-buffers-swapped-p) (set-buffer rmail-view-buffer))
-	    (setq cur (current-buffer))
-	    (save-restriction
-	      (widen)
-	      (with-temp-buffer
-		(insert-buffer-substring cur beg end)
-		(if babyl-format
-		    (rmail-output-as-babyl file-name noattribute)
-		  (rmail-output-as-mbox file-name noattribute)))))
+          (when (and rmail-output-reset-deleted-flag
+                     (rmail-message-deleted-p rmail-current-message))
+            (rmail-set-attribute rmail-deleted-attr-index nil)
+            (setq delete-attr-reset-p t))
+          ;; Make sure we undo our messing with the DELETED attribute.
+          (unwind-protect
+              (progn
+	        (setq beg (rmail-msgbeg rmail-current-message)
+		      end (rmail-msgend rmail-current-message))
+	        ;; All access to the buffer's local variables is now finished...
+	        (save-excursion
+	          ;; ... so it is ok to go to a different buffer.
+	          (if (rmail-buffers-swapped-p) (set-buffer rmail-view-buffer))
+	          (setq cur (current-buffer))
+	          (save-restriction
+	            (widen)
+	            (with-temp-buffer
+		      (insert-buffer-substring cur beg end)
+		      (if babyl-format
+		          (rmail-output-as-babyl file-name noattribute)
+		        (rmail-output-as-mbox file-name noattribute))))))
+            (if delete-attr-reset-p
+                (rmail-set-attribute rmail-deleted-attr-index t)))
 	  (or noattribute		; mark message as "filed"
 	      (rmail-set-attribute rmail-filed-attr-index t))
 	  (setq count (1- count))
 	  (let ((next-message-p
-		 (if rmail-delete-after-output
-		     (rmail-delete-forward)
-		   (if (> count 0)
-		       (rmail-next-undeleted-message 1))))
+                 (if rmail-output-reset-deleted-flag
+                     (progn
+                       (if rmail-delete-after-output
+                           (rmail-delete-message))
+                       (if (>= count 0)
+                           (let ((msgnum rmail-current-message))
+                             (rmail-next-message 1)
+                             (eq rmail-current-message (1+ msgnum)))))
+		   (if rmail-delete-after-output
+		       (rmail-delete-forward)
+		     (if (> count 0)
+		         (rmail-next-undeleted-message 1)))))
 		(num-appended (- orig-count count)))
 	    (if (and (> count 0) (not next-message-p))
 		(error "Only %d message%s appended" num-appended
@@ -531,6 +627,8 @@ so you should call `rmail-output' directly in that case."
     (if rmail-buffer
 	(set-buffer rmail-buffer)
       (error "There is no Rmail buffer"))
+    (if (zerop rmail-total-messages)
+	(error "No messages to output"))
     (let ((orig-count count)
 	  (cur (current-buffer)))
       (while (> count 0)
@@ -579,9 +677,9 @@ than appending to it.  Deletes the message after writing if
 	   (or (mail-fetch-field "Subject")
 	       rmail-default-body-file)))
      (setq default-file
-	   (replace-regexp-in-string ":" "-" default-file))
+	   (string-replace ":" "-" default-file))
      (setq default-file
-	   (replace-regexp-in-string " " "-" default-file))
+	   (string-replace " " "-" default-file))
      (list (setq rmail-default-body-file
 		 (read-file-name
 		  "Output message body to file: "
@@ -592,6 +690,8 @@ than appending to it.  Deletes the message after writing if
 	(expand-file-name file-name
 			  (and rmail-default-body-file
 			       (file-name-directory rmail-default-body-file))))
+  (if (zerop rmail-current-message)
+      (error "No message to output"))
   (save-excursion
     (goto-char (point-min))
     (search-forward "\n\n")
@@ -602,5 +702,4 @@ than appending to it.  Deletes the message after writing if
   (if rmail-delete-after-output
       (rmail-delete-forward)))
 
-;; arch-tag: 4059abf0-f249-4be4-8e0d-602d370d01d1
 ;;; rmailout.el ends here
