@@ -1,6 +1,6 @@
-;;; ruby-mode-tests.el --- Test suite for ruby-mode
+;;; ruby-mode-tests.el --- Test suite for ruby-mode  -*- lexical-binding:t -*-
 
-;; Copyright (C) 2012-2021 Free Software Foundation, Inc.
+;; Copyright (C) 2012-2024 Free Software Foundation, Inc.
 
 ;; This file is part of GNU Emacs.
 
@@ -22,6 +22,7 @@
 ;;; Code:
 
 (require 'ert)
+(require 'ert-x)
 (require 'ruby-mode)
 
 (defmacro ruby-with-temp-buffer (contents &rest body)
@@ -31,6 +32,13 @@
      (ruby-mode)
      ,@body))
 
+(defmacro ruby-with-temp-file (contents &rest body)
+  `(ruby-with-temp-buffer ,contents
+     (set-visited-file-name "ruby-mode-tests")
+     ,@body
+     (set-buffer-modified-p nil)
+     (delete-file buffer-file-name)))
+
 (defun ruby-should-indent (content column)
   "Assert indentation COLUMN on the last line of CONTENT."
   (ruby-with-temp-buffer content
@@ -149,6 +157,18 @@ VALUES-PLIST is a list with alternating index and value elements."
 (ert-deftest ruby-regexp-is-not-mistaken-for-slash-symbol ()
   (ruby-assert-state "x = /foo:/" 3 nil))
 
+(ert-deftest ruby-slash-not-regexp-when-surrounded-by-spaces ()
+  (ruby-assert-state "x = index / 3" 3 nil))
+
+(ert-deftest ruby-slash-not-regexp-when-no-spaces ()
+  (ruby-assert-state "x = index/3" 3 nil))
+
+(ert-deftest ruby-regexp-not-division-when-only-space-before ()
+  (ruby-assert-state "x = index /3" 3 ?/))
+
+(ert-deftest ruby-slash-not-regexp-when-only-space-after ()
+  (ruby-assert-state "x = index/ 3" 3 nil))
+
 (ert-deftest ruby-indent-simple ()
   (ruby-should-indent-buffer
    "if foo
@@ -349,7 +369,7 @@ VALUES-PLIST is a list with alternating index and value elements."
   (let ((ruby-align-chained-calls t))
     (ruby-should-indent-buffer
      "one.two.three
-     |       .four
+     |   .four
      |
      |my_array.select { |str| str.size > 5 }
      |        .map    { |str| str.downcase }"
@@ -399,6 +419,13 @@ VALUES-PLIST is a list with alternating index and value elements."
     (ruby-toggle-block)
     (should (string= "foo { \"#{bar}\" }" (buffer-string)))))
 
+(ert-deftest ruby-toggle-block-to-brace-no-space ()
+  (ruby-with-temp-buffer "foo do |b|\n  b + 2\nend"
+    (beginning-of-line)
+    (let (ruby-toggle-block-space-before-parameters)
+      (ruby-toggle-block))
+    (should (string= "foo {|b| b + 2 }" (buffer-string)))))
+
 (ert-deftest ruby-recognize-symbols-starting-with-at-character ()
   (ruby-assert-face ":@abc" 3 font-lock-constant-face))
 
@@ -496,7 +523,8 @@ VALUES-PLIST is a list with alternating index and value elements."
 (ert-deftest ruby-add-log-current-method-examples ()
   (let ((pairs '(("foo" . "#foo")
                  ("C.foo" . ".foo")
-                 ("self.foo" . ".foo"))))
+                 ("self.foo" . ".foo")
+                 ("<<" . "#<<"))))
     (dolist (pair pairs)
       (let ((name  (car pair))
             (value (cdr pair)))
@@ -521,9 +549,12 @@ VALUES-PLIST is a list with alternating index and value elements."
                           |    def foo
                           |    end
                           |    _
+                          |    def bar
+                          |    end
                           |  end
                           |end")
     (search-backward "_")
+    (delete-char 1)
     (should (string= (ruby-add-log-current-method)"M::C"))))
 
 (ert-deftest ruby-add-log-current-method-in-singleton-class ()
@@ -548,6 +579,22 @@ VALUES-PLIST is a list with alternating index and value elements."
     (search-backward "_")
     (should (string= (ruby-add-log-current-method) "C::D#foo"))))
 
+(ert-deftest ruby-add-log-current-method-singleton-referencing-outer ()
+  (ruby-with-temp-buffer (ruby-test-string
+                          "module M
+                          |  module N
+                          |    module C
+                          |      class D
+                          |        def C.foo
+                          |          _
+                          |        end
+                          |      end
+                          |    end
+                          |  end
+                          |end")
+    (search-backward "_")
+    (should (string= (ruby-add-log-current-method) "M::N::C.foo"))))
+
 (ert-deftest ruby-add-log-current-method-after-inner-class ()
   (ruby-with-temp-buffer (ruby-test-string
                           "module M
@@ -562,6 +609,45 @@ VALUES-PLIST is a list with alternating index and value elements."
     (search-backward "_")
     (should (string= (ruby-add-log-current-method) "M::C#foo"))))
 
+(ert-deftest ruby-add-log-current-method-after-inner-class-outside-methods ()
+  (ruby-with-temp-buffer (ruby-test-string
+                          "module M
+                          |  class C
+                          |    class D
+                          |    end
+                          |
+                          |_
+                          |  end
+                          |end")
+    (search-backward "_")
+    (delete-char 1)
+    (should (string= (ruby-add-log-current-method) "M::C"))))
+
+(ert-deftest ruby-add-log-current-method-after-inner-class-outside-methods-with-text ()
+  (ruby-with-temp-buffer (ruby-test-string
+                          "module M
+                          |  class C
+                          |    class D
+                          |    end
+                          |
+                          |    FOO = 5
+                          |  end
+                          |end")
+    (search-backward "FOO")
+    (should (string= (ruby-add-log-current-method) "M::C"))))
+
+(ert-deftest ruby-add-log-current-method-after-endless-method ()
+  (ruby-with-temp-buffer (ruby-test-string
+                          "module M
+                          |  class C
+                          |    def foo =
+                          |      4_
+                          |  end
+                          |end")
+    (search-backward "_")
+    (delete-char 1)
+    (should (string= (ruby-add-log-current-method) "M::C#foo"))))
+
 (defvar ruby-block-test-example
   (ruby-test-string
    "class C
@@ -711,7 +797,7 @@ VALUES-PLIST is a list with alternating index and value elements."
   (ruby-with-temp-buffer ruby-sexp-test-example
     (goto-char (point-min))
     (forward-line 1)
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (= 8 (line-number-at-pos)))))
 
 (ert-deftest ruby-backward-sexp-skips-method-calls-with-keyword-names ()
@@ -719,7 +805,7 @@ VALUES-PLIST is a list with alternating index and value elements."
     (goto-char (point-min))
     (forward-line 7)
     (end-of-line)
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (= 2 (line-number-at-pos)))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-no-args ()
@@ -728,7 +814,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do
      |end")
     (search-backward "do\n")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-backward-sexp-jumps-do-end-block-with-no-args ()
@@ -737,7 +823,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do
      |end")
     (goto-char (point-max))
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (looking-at "do$"))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-empty-args ()
@@ -746,7 +832,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do ||
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-backward-sexp-jumps-do-end-block-with-empty-args ()
@@ -755,7 +841,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do ||
      |end")
     (goto-char (point-max))
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (looking-at "do "))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-args ()
@@ -764,7 +850,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,b|
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-backward-sexp-jumps-do-end-block-with-args ()
@@ -773,7 +859,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,b|
      |end")
     (goto-char (point-max))
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (looking-at "do "))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-any-args ()
@@ -782,7 +868,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |*|
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-expanded-one-arg ()
@@ -791,7 +877,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,|
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-forward-sexp-jumps-do-end-block-with-one-and-any-args ()
@@ -800,7 +886,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,*|
      |end")
     (search-backward "do ")
-    (ruby-forward-sexp)
+    (forward-sexp)
     (should (eobp))))
 
 (ert-deftest ruby-backward-sexp-jumps-do-end-block-with-one-and-any-args ()
@@ -809,7 +895,7 @@ VALUES-PLIST is a list with alternating index and value elements."
      "proc do |a,*|
      |end")
     (goto-char (point-max))
-    (ruby-backward-sexp)
+    (backward-sexp)
     (should (looking-at "do "))))
 
 (ert-deftest ruby-toggle-string-quotes-quotes-correctly ()
@@ -842,6 +928,97 @@ VALUES-PLIST is a list with alternating index and value elements."
       (ruby--insert-coding-comment "utf-8")
       (should (string= "# encoding: utf-8\n\n" (buffer-string))))))
 
+(ert-deftest ruby--set-encoding-when-ascii ()
+  (ruby-with-temp-file "ascii"
+    (let ((ruby-encoding-magic-comment-style 'ruby)
+          (ruby-insert-encoding-magic-comment t))
+      (setq save-buffer-coding-system 'us-ascii)
+      (ruby-mode-set-encoding)
+      (should (string= "ascii" (buffer-string))))))
+
+(ert-deftest ruby--set-encoding-when-utf8 ()
+  (ruby-with-temp-file "💎"
+    (let ((ruby-encoding-magic-comment-style 'ruby)
+          (ruby-insert-encoding-magic-comment t))
+      (setq save-buffer-coding-system 'utf-8)
+      (ruby-mode-set-encoding)
+      (should (string= "💎" (buffer-string))))))
+
+(ert-deftest ruby--set-encoding-when-latin-15 ()
+  (ruby-with-temp-file "Ⓡ"
+    (let ((ruby-encoding-magic-comment-style 'ruby)
+          (ruby-insert-encoding-magic-comment t))
+      (setq save-buffer-coding-system 'iso-8859-15)
+      (ruby-mode-set-encoding)
+      (should (string= "# coding: iso-8859-15\nⓇ" (buffer-string))))))
+
+(ert-deftest ruby-imenu-with-private-modifier ()
+  (ruby-with-temp-buffer
+      (ruby-test-string
+       "class Blub
+       |  def hi
+       |    'Hi!'
+       |  end
+       |
+       |  def bye
+       |    'Bye!'
+       |  end
+       |
+       |  private def hiding
+       |    'You can't see me'
+       |  end
+       |end")
+    (should (equal (mapcar #'car (ruby-imenu-create-index))
+                   '("Blub"
+                     "Blub#hi"
+                     "Blub#bye"
+                     "Blub#hiding")))))
+
+(defmacro ruby-deftest-indent (file)
+  `(ert-deftest ,(intern (format "ruby-indent-test/%s" file)) ()
+     ;; :tags '(:expensive-test)
+     (let ((buf (find-file-noselect (ert-resource-file ,file))))
+       (unwind-protect
+           (with-current-buffer buf
+             (let ((orig (buffer-string)))
+               ;; Indent and check that we get the original text.
+               (indent-region (point-min) (point-max))
+               (should (equal (buffer-string) orig))))
+         (kill-buffer buf)))))
+
+(ruby-deftest-indent "ruby.rb")
+(ruby-deftest-indent "ruby-after-operator-indent.rb")
+(ruby-deftest-indent "ruby-block-indent.rb")
+(ruby-deftest-indent "ruby-method-call-indent.rb")
+(ruby-deftest-indent "ruby-method-params-indent.rb")
+(ruby-deftest-indent "ruby-parenless-call-arguments-indent.rb")
+
+(ert-deftest ruby--test-chained-indentation ()
+  (with-temp-buffer
+    (ruby-mode)
+    (setq-local ruby-align-chained-calls t)
+    (insert "some_variable.where
+.not(x: nil)
+.where(y: 2)
+")
+    (indent-region (point-min) (point-max))
+    (should (equal (buffer-string)
+                   "some_variable.where
+             .not(x: nil)
+             .where(y: 2)
+")))
+
+  (with-temp-buffer
+    (ruby-mode)
+    (setq-local ruby-align-chained-calls t)
+    (insert "some_variable.where.not(x: nil)
+.where(y: 2)
+")
+    (indent-region (point-min) (point-max))
+    (should (equal (buffer-string)
+                   "some_variable.where.not(x: nil)
+             .where(y: 2)
+"))))
 
 (provide 'ruby-mode-tests)
 
